package main

import (
	"os"
	"os/user"
	"path"

	"gopkg.in/yaml.v2"
)

type configPathsStruct struct {
	argument string
	current  string
	user     string
	main     string
}

var (
	configPaths = configPathsStruct{
		main: "/etc/requirements-manager/config.yaml",
	}
)

// Configuration
type Configuration struct {
	Path string `yaml:"Path"`

	Locale struct {
		DateFormat string `yaml:"DateFormat"`
	} `yaml:"Locale"`

	Gitlab struct {
		Token string `yaml:"Token"`
	} `yaml:"Gitlab"`

	Overview struct {
		Template struct {
			Markdown string `yaml:"Markdown"`
			Latex    string `yaml:"Latex"`
			Internal struct {
				MarkdownContent string
			}
		} `yaml:"Template"`
	} `yaml:"Overview"`

	Requirement struct {
		Table struct {
			Columns []string `yaml:"Columns"`
		} `yaml:"Table"`
		Template struct {
			Add string `yaml:"Add"`
		} `yaml:"Template"`
	} `yaml:"Requirement"`
}

func fileExists(name string) (found bool) {
	if f, err := os.Stat(name); err == nil {
		return f.Mode().IsRegular()
	}
	return false
}

// NewConfiguration
func NewConfiguration(argConfigPath string) *Configuration {

	current, err := os.Getwd()
	if err != nil {
		panic(err)
	}

	filename := path.Clean(current + "/config.yaml")

	cfg := Configuration{}

	usr, err := user.Current()
	if err == nil {
		configPaths.user = usr.HomeDir + "/.config/requirements-manager/config.yaml"
	}

	if argConfigPath != "" {
		configPaths.argument = argConfigPath
	}

	switch {
	case fileExists(configPaths.argument):
		readFile(&cfg, configPaths.argument)
	case fileExists(filename):
		readFile(&cfg, filename)
	case fileExists(current):
		readFile(&cfg, current)
	case fileExists(configPaths.user):
		readFile(&cfg, configPaths.user)
	case fileExists(configPaths.main):
		readFile(&cfg, configPaths.main)
	}

	return &cfg
}

func readFile(cfg *Configuration, filename string) {
	f, err := os.Open(filename)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}

	defer f.Close()

	decoder := yaml.NewDecoder(f)
	err = decoder.Decode(cfg)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}
}
